BEGIN;

-- 1.0pre5 -> jmd-1.0pre6

SET ROLE :db_adm;

INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('jmd-1.0pre6/03-fallakte.sql', current_timestamp);

--ISSUE850
UPDATE master_tbl set weitere_akt_anzahl_informationsveranst = NULL;
UPDATE master_tbl set weitere_akt_anzahl_workshops = NULL;
UPDATE master_tbl set weitere_akt_anzahl_trainings = NULL;
UPDATE master_tbl set weitere_akt_anzahl_veroeffentlichungen = NULL;
UPDATE master_tbl set weitere_akt_anzahl_sonstiges = NULL;

--ISSUE849
INSERT INTO institution_art_tbl (id, value) VALUES (57, 'Förderschulen');

--ISSUE848
UPDATE rg_kooperationsbezug_tbl set institution_art = 11 where institution_art
= 12;
DELETE FROM institution_art_tbl where id = 12;

--ISSUE1149
INSERT INTO raeumliche_zustaendigkeit_tbl (id, value) VALUES (-2, 'Sonstiges');
INSERT INTO raeumliche_zustaendigkeit_tbl (id, value) VALUES (6, 'ein Landkreis und eine kreisfreie Stadt');
ALTER TABLE master_tbl ADD COLUMN raeumliche_zustaendigkeit_nennung VARCHAR(60);

-- ISSUE1150
ALTER TABLE master_tbl ADD COLUMN existenz_jugendhilfekonferenz INTEGER DEFAULT -1;
ALTER TABLE master_tbl ADD COLUMN existenz_jugendkonferenz INTEGER DEFAULT -1;
ALTER TABLE master_tbl ADD FOREIGN KEY (existenz_jugendhilfekonferenz) REFERENCES erstellung_netzwerkkarte_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (existenz_jugendkonferenz) REFERENCES erstellung_netzwerkkarte_tbl (id); 

-- ISSUE1151
-- Anlegen der Tabelle für die RG
CREATE TABLE rg_netzwerk_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    master_id INTEGER NOT NULL,
    uuid_id CHAR(36),
    zielgruppen_netzwerk INTEGER DEFAULT -1,
    zielgruppen_netzwerk_nennung VARCHAR(60),
    mitarbeit_netzwerk INTEGER DEFAULT -1,
    mitarbeit_netzwerk_erlaueterung VARCHAR(60),
    erstellung_netzwerkkarte INTEGER DEFAULT -1,
    beteiligung_integrationskonzepte INTEGER DEFAULT -1,
    beteiligung_integrationskonzepte_erlaeuterung VARCHAR(60),
    FOREIGN KEY (master_id) REFERENCES master_tbl (id) ON DELETE CASCADE,
    UNIQUE (uuid_id),
    FOREIGN KEY (zielgruppen_netzwerk) REFERENCES zielgruppen_netzwerk_tbl (id),
    FOREIGN KEY (mitarbeit_netzwerk) REFERENCES mitarbeit_netzwerk_tbl (id),
    FOREIGN KEY (erstellung_netzwerkkarte) REFERENCES erstellung_netzwerkkarte_tbl (id),
    FOREIGN KEY (beteiligung_integrationskonzepte) REFERENCES beteiligung_integrationskonzepte_tbl (id)
);
-- Anlegen der Funktionen für de RG
-- PLPython is an untrusted language. -> Need to be postgres.
SET ROLE postgres;

-- DROP FUNCTION get_case_structure(int4);
CREATE OR REPLACE FUNCTION get_case_structure(case_id int4) RETURNS TEXT AS $$

class Node(object):

    def __init__(self, name, id = None, children = []):
        self.name     = name
        self.id       = id
        self.children = children

    def to_json(self, out):
        out.append('{"name":"%s","id":%d' % (self.name, self.id))
        if self.children:
            out.append(',"children":[')
            first = True
            for child in self.children:
                if first: first = False
                else:     out.append(",")
                child.to_json(out)
            out.append("]")
        out.append("}")

    def recursive_build(self, mid, children):
        select = "SELECT id FROM %s WHERE master_id = %d" % (
            self.name, mid)
        ids = [r['id'] for r in plpy.execute(select)]
        for id in ids:
            sub_children = []
            node = Node(self.name, id, sub_children)
            for child in self.children:
                child.recursive_build(id, sub_children)
            children.append(node)

TREE_STRUCTURE = Node('master_tbl', children = [
    Node('rg_netzwerk_tbl'),
    Node('rg_kooperationsbezug_tbl'),
    Node('rg_kjp_gruppenangebote_tbl'),
    Node('rg_nichtkjp_gruppenangebote_tbl')
])

r = plpy.execute(
    "SELECT modified, cache FROM case_structure "
    "WHERE master_id = %d" % case_id, 1)

if not r: return ""
r = r[0]

if r['modified']: # need refresh
    children = []
    tree = Node(TREE_STRUCTURE.name, case_id, children)
    for child in TREE_STRUCTURE.children:
        child.recursive_build(case_id, children)
    out = []
    tree.to_json(out)
    cache = "".join(out)
    plan = plpy.prepare(
        "UPDATE case_structure SET modified = false, "
        "cache = $1 WHERE master_id = $2", ["text", "int4"])
    plpy.execute(plan, [cache, case_id])
    return cache

return r['cache']

$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
-- Continue as db-owner from here
ALTER FUNCTION get_case_structure(int4) OWNER TO :adm_ka_owner;
SET ROLE :adm_ka_owner;

CREATE OR REPLACE FUNCTION rg_netzwerk_tbl_insert_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure SET modified = true WHERE master_id
    IN (
        SELECT master_tbl.id FROM rg_netzwerk_tbl
        INNER JOIN master_tbl ON rg_netzwerk_tbl.master_id = master_tbl.id
        WHERE rg_netzwerk_tbl.id = NEW.id
    );
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_netzwerk_tbl_insert_trigger AFTER INSERT ON rg_netzwerk_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_netzwerk_tbl_insert_func();

CREATE OR REPLACE FUNCTION rg_netzwerk_tbl_delete_func() RETURNS TRIGGER AS $$
BEGIN
    UPDATE case_structure SET modified = true WHERE master_id
    IN (
        SELECT master_tbl.id FROM rg_netzwerk_tbl
        INNER JOIN master_tbl ON rg_netzwerk_tbl.master_id = master_tbl.id
        WHERE rg_netzwerk_tbl.id = OLD.id
    );
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_netzwerk_tbl_delete_trigger BEFORE DELETE ON rg_netzwerk_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_netzwerk_tbl_delete_func();

CREATE OR REPLACE FUNCTION create_rg_netzwerk_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        --RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_netzwerk_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_netzwerk_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_netzwerk_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_netzwerk_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;

CREATE OR REPLACE FUNCTION delete_rg_netzwerk_tbl(rid integer)
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_netzwerk_tbl WHERE id = rid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    DELETE FROM rg_netzwerk_tbl WHERE id = rid;
END;
$$;

-- Anlegen eines Eintrags
-- Übernahme der alten Einträge in eine RG
-- Es wird in jedem Fall eine RG angelegt
CREATE OR REPLACE FUNCTION insert_rg() RETURNS VOID AS $$
DECLARE
  r RECORD;
  rgid INTEGER;
BEGIN
  FOR r in SELECT id, zielgruppen_netzwerk, zielgruppen_netzwerk_nennung,
  mitarbeit_netzwerk, mitarbeit_netzwerk_erlaueterung,
  erstellung_netzwerkkarte, beteiligung_integrationskonzepte,
  beteiligung_integrationskonzepte_erlaeuterung  
  FROM master_tbl LOOP
      -- Insert new RG
      rgid = create_rg_netzwerk_tbl(r.id, NULL);
      ---- Update values in RG
      UPDATE rg_netzwerk_tbl set 
      zielgruppen_netzwerk = r.zielgruppen_netzwerk, 
      zielgruppen_netzwerk_nennung = r.zielgruppen_netzwerk_nennung,
      mitarbeit_netzwerk = r.mitarbeit_netzwerk,
      mitarbeit_netzwerk_erlaueterung = r.mitarbeit_netzwerk_erlaueterung,
      erstellung_netzwerkkarte = r.erstellung_netzwerkkarte,
      beteiligung_integrationskonzepte = r.beteiligung_integrationskonzepte,
      beteiligung_integrationskonzepte_erlaeuterung = r.beteiligung_integrationskonzepte_erlaeuterung
      where id = rgid;
      RAISE NOTICE 'rgid is %', rgid;
  END LOOP;
END;
$$ LANGUAGE plpgsql;
SELECT insert_rg();
DROP FUNCTION insert_rg();

CREATE OR REPLACE FUNCTION create_rg_netzwerk_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_netzwerk_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_netzwerk_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_netzwerk_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_netzwerk_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;


CREATE OR REPLACE FUNCTION create_master_tbl(nuuid char(36))
    RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    standin integer;
    rid     integer;
BEGIN
    rid := create_master_ds(nuuid);
    IF nuuid IS NULL THEN
        PERFORM create_rg_netzwerk_tbl(rid, NULL);
    END IF;

    RETURN rid;
END;
$$;

REVOKE ALL ON FUNCTION create_master_tbl(char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_master_tbl(char(36))
    TO GROUP :cm_group;

ALTER TABLE master_tbl DROP zielgruppen_netzwerk CASCADE;
ALTER TABLE master_tbl DROP zielgruppen_netzwerk_nennung CASCADE;
ALTER TABLE master_tbl DROP mitarbeit_netzwerk CASCADE;
ALTER TABLE master_tbl DROP mitarbeit_netzwerk_erlaueterung CASCADE;
ALTER TABLE master_tbl DROP erstellung_netzwerkkarte CASCADE;
ALTER TABLE master_tbl DROP beteiligung_integrationskonzepte CASCADE;
ALTER TABLE master_tbl DROP beteiligung_integrationskonzepte_erlaeuterung CASCADE;

-- VIEWS
-- views for master table and repeat groups
--
CREATE OR REPLACE VIEW master_tbl_view
AS SELECT master_tbl.* FROM master_tbl
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE master_tbl_update_rule AS ON UPDATE TO master_tbl_view
DO INSTEAD UPDATE master_tbl SET
    phase = NEW.phase,
    datum_begin_zeitraum = NEW.datum_begin_zeitraum,
    datum_ende_zeitraum = NEW.datum_ende_zeitraum,
    raeumliche_zustaendigkeit = NEW.raeumliche_zustaendigkeit,
    raeumliche_zustaendigkeit_nennung = NEW.raeumliche_zustaendigkeit_nennung,
    nennung_zustaendigkeit = NEW.nennung_zustaendigkeit,
    jmdstruktur = NEW.jmdstruktur,
    anzahl_ha_stellenteile = NEW.anzahl_ha_stellenteile,
    anzahl_ha_personen = NEW.anzahl_ha_personen,
    anzahl_ha_maennlich = NEW.anzahl_ha_maennlich,
    anzahl_ha_weiblich = NEW.anzahl_ha_weiblich,
    anzahl_ha_migration = NEW.anzahl_ha_migration,
    ha_sprachkenntnis_arabisch = NEW.ha_sprachkenntnis_arabisch,
    ha_sprachkenntnis_englisch = NEW.ha_sprachkenntnis_englisch,
    ha_sprachkenntnis_franzoesisch = NEW.ha_sprachkenntnis_franzoesisch,
    ha_sprachkenntnis_polnisch = NEW.ha_sprachkenntnis_polnisch,
    ha_sprachkenntnis_russisch = NEW.ha_sprachkenntnis_russisch,
    ha_sprachkenntnis_spanisch = NEW.ha_sprachkenntnis_spanisch,
    ha_sprachkenntnis_tuerkisch = NEW.ha_sprachkenntnis_tuerkisch,
    ha_sprachkenntnis_sonstige = NEW.ha_sprachkenntnis_sonstige,
    ha_sprachkenntnisse_sonstige_nennung = NEW.ha_sprachkenntnisse_sonstige_nennung,
    verteilung_aa_begleitung_cm = NEW.verteilung_aa_begleitung_cm,
    verteilung_aa_betreuung = NEW.verteilung_aa_betreuung,
    verteilung_aa_gruppenangebote = NEW.verteilung_aa_gruppenangebote,
    verteilung_aa_netzwerkarbeit = NEW.verteilung_aa_netzwerkarbeit,
    verteilung_aa_oeffentlichkeit = NEW.verteilung_aa_oeffentlichkeit,
    verteilung_aa_evaluation = NEW.verteilung_aa_evaluation,
    verteilung_aa_dokumentation = NEW.verteilung_aa_dokumentation,
    verteilung_aa_sonstiges = NEW.verteilung_aa_sonstiges,
    verteilung_aa_sonstiges_nennung = NEW.verteilung_aa_sonstiges_nennung,
    anzahl_nha_stellenteile = NEW.anzahl_nha_stellenteile,
    anzahl_nha_personen = NEW.anzahl_nha_personen,
    anzahl_nha_maennlich = NEW.anzahl_nha_maennlich,
    anzahl_nha_weiblich = NEW.anzahl_nha_weiblich,
    anzahl_nha_migration = NEW.anzahl_nha_migration,
    anzahl_nha_stundenunfang = NEW.anzahl_nha_stundenunfang,
    anzahl_ea_personen = NEW.anzahl_ea_personen,
    anzahl_ea_maennlich = NEW.anzahl_ea_maennlich,
    anzahl_ea_weiblich = NEW.anzahl_ea_weiblich,
    anzahl_ea_migration = NEW.anzahl_ea_migration,
    anzahl_gesamt_personen = NEW.anzahl_gesamt_personen,
    nennung_qualimanagement = NEW.nennung_qualimanagement,
    existenz_jugendhilfekonferenz = NEW.existenz_jugendhilfekonferenz,
    teilnahme_jugenhilfekonferenz = NEW.teilnahme_jugenhilfekonferenz,
    existenz_jugendkonferenz = NEW.existenz_jugendkonferenz,
    teilnahme_jugenkonferenz = NEW.teilnahme_jugenkonferenz,
    mitarbeit_gremien = NEW.mitarbeit_gremien,
    weiter_akt_art_interkulturell = NEW.weiter_akt_art_interkulturell,
    weiter_akt_art_aufsuchende = NEW.weiter_akt_art_aufsuchende,
    weiter_akt_art_eltern = NEW.weiter_akt_art_eltern,
    weiter_akt_art_freiwilligerdienst = NEW.weiter_akt_art_freiwilligerdienst,
    weiter_akt_art_sonstiges = NEW.weiter_akt_art_sonstiges,
    weitere_akt_anzahl_informationsveranst = NEW.weitere_akt_anzahl_informationsveranst,
    weitere_akt_anzahl_workshops = NEW.weitere_akt_anzahl_workshops,
    weitere_akt_anzahl_trainings = NEW.weitere_akt_anzahl_trainings,
    weitere_akt_anzahl_veroeffentlichungen = NEW.weitere_akt_anzahl_veroeffentlichungen,
    weitere_akt_anzahl_sonstiges = NEW.weitere_akt_anzahl_sonstiges
WHERE id = NEW.id;

CREATE OR REPLACE RULE master_tbl_delete_rule AS ON DELETE TO master_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_insert_rule AS ON INSERT TO master_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON master_tbl_view TO GROUP :cm_group;
GRANT SELECT ON master_tbl_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW master_tbl_anonym_view
AS SELECT
    id,
    uuid_id,
    bearbeiter_id,
    fkz,
    phase,
    datum_begin_zeitraum,
    datum_ende_zeitraum,
    raeumliche_zustaendigkeit,
    raeumliche_zustaendigkeit_nennung,
    nennung_zustaendigkeit,
    jmdstruktur,
    anzahl_ha_stellenteile,
    anzahl_ha_personen,
    anzahl_ha_maennlich,
    anzahl_ha_weiblich,
    anzahl_ha_migration,
    ha_sprachkenntnis_arabisch,
    ha_sprachkenntnis_englisch,
    ha_sprachkenntnis_franzoesisch,
    ha_sprachkenntnis_polnisch,
    ha_sprachkenntnis_russisch,
    ha_sprachkenntnis_spanisch,
    ha_sprachkenntnis_tuerkisch,
    ha_sprachkenntnis_sonstige,
    ha_sprachkenntnisse_sonstige_nennung,
    verteilung_aa_begleitung_cm,
    verteilung_aa_betreuung,
    verteilung_aa_gruppenangebote,
    verteilung_aa_netzwerkarbeit,
    verteilung_aa_oeffentlichkeit,
    verteilung_aa_evaluation,
    verteilung_aa_dokumentation,
    verteilung_aa_sonstiges,
    verteilung_aa_sonstiges_nennung,
    anzahl_nha_stellenteile,
    anzahl_nha_personen,
    anzahl_nha_maennlich,
    anzahl_nha_weiblich,
    anzahl_nha_migration,
    anzahl_nha_stundenunfang,
    anzahl_ea_personen,
    anzahl_ea_maennlich,
    anzahl_ea_weiblich,
    anzahl_ea_migration,
    anzahl_gesamt_personen,
    nennung_qualimanagement,
    existenz_jugendhilfekonferenz,
    teilnahme_jugenhilfekonferenz,
    existenz_jugendkonferenz,
    teilnahme_jugenkonferenz,
    mitarbeit_gremien,
    weiter_akt_art_interkulturell,
    weiter_akt_art_aufsuchende,
    weiter_akt_art_eltern,
    weiter_akt_art_freiwilligerdienst,
    weiter_akt_art_sonstiges,
    weitere_akt_anzahl_informationsveranst,
    weitere_akt_anzahl_workshops,
    weitere_akt_anzahl_trainings,
    weitere_akt_anzahl_veroeffentlichungen,
    weitere_akt_anzahl_sonstiges FROM master_tbl_view;

CREATE OR REPLACE RULE master_tbl_anonym_delete_rule AS ON DELETE TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE master_tbl_anonym_insert_rule AS ON INSERT TO master_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON master_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON master_tbl_anonym_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_netzwerk_tbl_view
AS SELECT rg_netzwerk_tbl.* FROM rg_netzwerk_tbl
    INNER JOIN master_tbl
    ON         rg_netzwerk_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE rg_netzwerk_tbl_update_rule AS ON UPDATE TO rg_netzwerk_tbl_view
DO INSTEAD UPDATE rg_netzwerk_tbl SET
    zielgruppen_netzwerk = NEW.zielgruppen_netzwerk,
    zielgruppen_netzwerk_nennung = NEW.zielgruppen_netzwerk_nennung,
    mitarbeit_netzwerk = NEW.mitarbeit_netzwerk,
    mitarbeit_netzwerk_erlaueterung = NEW.mitarbeit_netzwerk_erlaueterung,
    erstellung_netzwerkkarte = NEW.erstellung_netzwerkkarte,
    beteiligung_integrationskonzepte = NEW.beteiligung_integrationskonzepte,
    beteiligung_integrationskonzepte_erlaeuterung = NEW.beteiligung_integrationskonzepte_erlaeuterung
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_netzwerk_tbl_delete_rule AS ON DELETE TO rg_netzwerk_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_netzwerk_tbl_insert_rule AS ON INSERT TO rg_netzwerk_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_netzwerk_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON rg_netzwerk_tbl_view TO GROUP :cm_group;
GRANT SELECT ON rg_netzwerk_tbl_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_netzwerk_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    zielgruppen_netzwerk,
    zielgruppen_netzwerk_nennung,
    mitarbeit_netzwerk,
    mitarbeit_netzwerk_erlaueterung,
    erstellung_netzwerkkarte,
    beteiligung_integrationskonzepte,
    beteiligung_integrationskonzepte_erlaeuterung FROM rg_netzwerk_tbl_view;

CREATE OR REPLACE RULE rg_netzwerk_tbl_anonym_delete_rule AS ON DELETE TO rg_netzwerk_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_netzwerk_tbl_anonym_insert_rule AS ON INSERT TO rg_netzwerk_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_netzwerk_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_netzwerk_tbl_anonym_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_netzwerk_tbl_view
AS SELECT rg_netzwerk_tbl.* FROM rg_netzwerk_tbl
    INNER JOIN master_tbl
    ON         rg_netzwerk_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2 
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE rg_netzwerk_tbl_update_rule AS ON UPDATE TO rg_netzwerk_tbl_view
DO INSTEAD UPDATE rg_netzwerk_tbl SET
    zielgruppen_netzwerk = NEW.zielgruppen_netzwerk,
    zielgruppen_netzwerk_nennung = NEW.zielgruppen_netzwerk_nennung,
    mitarbeit_netzwerk = NEW.mitarbeit_netzwerk,
    mitarbeit_netzwerk_erlaueterung = NEW.mitarbeit_netzwerk_erlaueterung,
    erstellung_netzwerkkarte = NEW.erstellung_netzwerkkarte,
    beteiligung_integrationskonzepte = NEW.beteiligung_integrationskonzepte,
    beteiligung_integrationskonzepte_erlaeuterung = NEW.beteiligung_integrationskonzepte_erlaeuterung
WHERE id = NEW.id;

CREATE OR REPLACE RULE rg_netzwerk_tbl_delete_rule AS ON DELETE TO rg_netzwerk_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_netzwerk_tbl_insert_rule AS ON INSERT TO rg_netzwerk_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_netzwerk_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON rg_netzwerk_tbl_view TO GROUP :cm_group;
GRANT SELECT ON rg_netzwerk_tbl_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_netzwerk_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    zielgruppen_netzwerk,
    zielgruppen_netzwerk_nennung,
    mitarbeit_netzwerk,
    mitarbeit_netzwerk_erlaueterung,
    erstellung_netzwerkkarte,
    beteiligung_integrationskonzepte,
    beteiligung_integrationskonzepte_erlaeuterung FROM rg_netzwerk_tbl_view;

CREATE OR REPLACE RULE rg_netzwerk_tbl_anonym_delete_rule AS ON DELETE TO rg_netzwerk_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_netzwerk_tbl_anonym_insert_rule AS ON INSERT TO rg_netzwerk_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_netzwerk_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_netzwerk_tbl_anonym_view TO GROUP :admin_group;

CREATE OR REPLACE  VIEW master_tbl_eval_total_view AS
    SELECT m.* FROM master_tbl m JOIN ka_status_tbl s
        ON m.id = s.master_id
        WHERE
            s.status <> 3 AND
            (m.bearbeiter_id = (getuserid())::integer)
            OR pg_has_role('admin','MEMBER');

GRANT SELECT ON master_tbl_eval_total_view TO :cm_group;
GRANT SELECT ON master_tbl_eval_total_view TO :admin_group;

--
-- Name: master_tbl_eval_incm_view; Type: VIEW; Schema: public; Owner: postgres
--

CREATE OR REPLACE  VIEW master_tbl_eval_incm_view AS
    SELECT * FROM master_tbl_eval_total_view
        WHERE phase IN (1);

GRANT SELECT ON public.master_tbl_eval_incm_view TO :cm_group;
GRANT SELECT ON public.master_tbl_eval_incm_view TO :admin_group;

CREATE OR REPLACE  VIEW master_tbl_incm_view AS
    SELECT * FROM master_tbl_view
        WHERE phase IN (1);

GRANT SELECT ON public.master_tbl_incm_view TO :cm_group;
GRANT SELECT ON public.master_tbl_incm_view TO :admin_group;

-- Finish
SET ROLE postgres;

COMMIT;
