-- Upgrade script for the changes to create_master_ds and
-- initial_master_ds_owner_id.

CREATE OR REPLACE FUNCTION initial_master_ds_owner_id() RETURNS integer
    LANGUAGE 'plpgsql' STABLE
    AS $$
BEGIN
    RETURN getuserid();
END;
$$;

REVOKE ALL ON FUNCTION initial_master_ds_owner_id() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION initial_master_ds_owner_id() TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION initial_master_ds_owner_id() TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION create_master_ds(nuuid char(36))
    RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    ownerid integer;
    standin integer;
    mid     integer;
BEGIN
    ownerid := initial_master_ds_owner_id();
    INSERT INTO master_tbl (bearbeiter_id) VALUES (ownerid);
    SELECT INTO standin vertreter from ka_benutzer_tbl WHERE id = ownerid;

    IF standin IS NOT NULL THEN
        INSERT INTO nm_benutzer_master_tbl(benutzer_id, master_id)
                VALUES (standin, currval('master_tbl_id_seq'));
    END IF;

    SELECT INTO mid currval('master_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE master_tbl SET uuid_id = nuuid WHERE id = mid;
    ELSE
        PERFORM set_uuid('master_tbl', mid);
    END IF;

    PERFORM create_status_ds(mid);
    PERFORM create_meta_ds(mid);
    RETURN mid;
END;
$$;

REVOKE ALL ON FUNCTION create_master_ds(char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_master_ds(char(36))
    TO GROUP :cm_group;
