# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging
import datetime

from pylons import app_globals as g, url
from pylons.controllers.util import redirect

import mpulsweb.lib.helpers as h
from mpulsweb.lib.dialogs import checkPrivacyStatement, checkSyncPermission, \
    checkCaseLinked
from mpulsweb.lib.security import checkRole
from mpulsweb.lib.base import session, restrict_to_POST
from mpulsweb.controllers.case import CaseController as MpulsCaseController
from mpulsweb.model.agencysettings import Agency

from jmdweb.controllers.formularpage import JMD2StartRendererController
from jmdweb.lib.validators import NewCaseForm


log = logging.getLogger(__name__)


class CaseController(JMD2StartRendererController, MpulsCaseController):

    def _get_validator(self):
        return NewCaseForm()

    def _meta_tree_for_imports(self):
        return g.formed_tree_for_imports

    def _add_custom_fields(self):
        fields = {}

        agency = Agency()
        name = agency.getName()
        if name:
            fields['jmd_name'] = name

        return fields

    # Completely overwrite the _render_new method to unset
    # htmlfill force_defaults.
    def _render_new(self, session_case, form_defaults=None, form_errors=None):
        if form_defaults is None:
            form_defaults = session_case.getData()
        if not form_defaults.has_key("datum_ea_start"):
            form_defaults["datum_ea_start"] = h.format_date(
                                                    datetime.date.today())
        if form_errors is None:
            form_errors = {}

        from pylons import tmpl_context as c
        c.case = session_case

        from mpulsweb.lib.base import render, render_htmlfill
        form = render('/casemanagement/new.mako')
        return render_htmlfill(form, defaults=form_defaults,
                               force_defaults=False,
                               auto_insert_errors=False, errors=form_errors)


    @checkRole('cm')
    @restrict_to_POST
    def newAction(self):
        """Extend newAction to print also the discretion statement."""

        from pylons import request, tmpl_context as c
        from mpulsweb.model.case import SessionCase
        from mpulsweb.lib.base import session, render
        from jmdweb.model.statement import DiscretionStatement

        session_case = SessionCase()
        session_case.setData(request.params)

        session['case'] = session_case
        # Use the session to store the folding information
        session['folded.EinwilligungenMuster'] = \
                    request.params.get('folded.EinwilligungenMuster')
        session['folded.SchweigepflichtMuster'] = \
                    request.params.get('folded.SchweigepflichtMuster')
        session.save()

        # Do we want to print the discretion statement?
        if "printDiscretion" in request.params:
            c.print_version = 1
            statement = DiscretionStatement(session_case, Agency())
            c.content = statement.fillout()
            return render('privacy/default_statement.mako')

        else:
            return super(CaseController,
                     self).newAction()

    @checkRole(('admin', 'cm', 'pt_dlr'))
    # NOTE: checkPrivacyStatement calls self._loadCase which updates the
    # session state including the current case id and the navigation
    # tree. The select method implementation below relies on this!
    @checkPrivacyStatement()
    @checkSyncPermission()
    @checkCaseLinked()
    def select(self, id, confirmed=0):
        """Overwrite base method to redirect to first form page."""
        navigation = session['navigation.tree']
        redirect(url(controller="navigation", action="select_branch",
                     id=navigation.first_page.key))


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
