# -*- coding: utf-8 -*-
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.

from formencode import ForEach
from formencode.validators import Bool, String, Int

from mpulsweb.lib.validators import BaseFormValidator


class NewCaseForm(BaseFormValidator):

    name = String()
    vorname = String()
    adresse_strasse = String()
    adresse_strassenr = String()
    adresse_ort = String()
    adresse_plz = String()
    statement_signed = Bool(if_missing=False)
    sync_permission = Bool(if_missing=False)

    # Fields required for phase "Eingangsdaten"
    # These are converted and validated via libformed, so they are
    # strings in the form as far as formencode is concernded
    datum_ea_start = String(not_empty=True)

    art_begleitung = String(if_missing="-1")
    einwilligung = String(if_missing="-1")


class PermissionForm(BaseFormValidator):

    userid = ForEach(Int(), convert_to_list=True)


class EditSettingsForm(BaseFormValidator):

    # We do not allow any fields that are not explicitly listed to make
    # sure that users cannot try to modify settings stored in
    # ka_konfiguration_tbl that are not listed here.
    allow_extra_fields=False
    filter_extra_fields=True

    ka_name = String()
    ka_adresse = String()

    anon_transfer = String(if_missing='off')
    meta_transfer = String(if_missing='off')


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8
