# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import logging

from mpulsweb.model.agencysettings import Agency
from mpulsweb.model.case import MpulsCase, MpulsCaseFactory, \
     MpulsCaseOverview, MpulsCaseBundle, MpulsCaseState
from mpulsweb.model.state import PhaseStateMachine

from jmdweb.model.statement import DiscretionStatement


log = logging.getLogger(__name__)



class CaseBundle(MpulsCaseBundle):

    pass


class CaseOverview(MpulsCaseOverview):

    pass


class State(MpulsCaseState):

    pass


class CaseFactory(MpulsCaseFactory):

    def __init__(self, case_cls=None):
        if case_cls is None:
            # FIXME Case is not defined or imported here. Should it be a
            # MpulsCase???
            case_cls = Case
        self.case_cls = case_cls

    def get_state_machine(self):
        return JMDPhaseStateMachine()


class Case(MpulsCase):

    def getHashFields(self):
        """Returns a dictionary of fields which is used to build the hashkey
        for the meta server"""
        fields = {}
        fields['firstname'] = self.get_value('vorname')
        fields['birthname'] = self.get_value('name')
        fields['gender'] = self.get_value('geschlecht')
        fields['birthdate'] = self.get_value('geburtsdatum')
        return fields

    def getDiscretionStatement(self):
        self.discretion_statement = None
        """Returns the discretion statement of the case"""
        if not self.discretion_statement is None:
            return self.discretion_statement
        try:
            # Create privacy_statement
            self.discretion_statement = DiscretionStatement(self, Agency())
            return self.discretion_statement
        except:
            self.discretion_statement = None
            raise
        return None



class JMDPhaseStateMachine(PhaseStateMachine):

     def _load_case(self, case):
         """Load case and update the phase list for non-jmd2start agencies.
         This method extends the base class method to remove the
         jmd2start specific fields from the phaselist for non-jmd2start
         agencies after calling the base class method. This is done in
         order not to show those fields in the phase overview where they
         would be confusing for users of non-jmd2start agencies.
         """

         super(JMDPhaseStateMachine, self)._load_case(case)
         if not Agency().get_value("jmd2start"):
             remove_jmd2start_fields(self.phaselist)


def remove_jmd2start_fields(phaselist):
    """Removes the jmd2start specific fields from phaselist.
    phaselist is modified in-place.
    """
    phase = phaselist.get_phase(1)
    for i in range(len(phase.fields) - 1, -1, -1):
        if (phase.fields[i].name.partition(":")[0] in ("zielgruppe",
                                                       "aufenthaltspapier",
                                                       "aufenthaltspapier_eintritt")):
            del phase.fields[i]



# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
