# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2010 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
# Authors:
# Roland Geider <roland.geider@intevation.de>
# Ludwig Reiter <ludwig.reiter@intevation.de>

import logging

from datetime import date

from pylons import app_globals as g

from mpulsweb.lib.db import db
from mpulsweb.model.agencysettings import Agency


log = logging.getLogger(__name__)

COUNT_ALL_CASES = """
SELECT DISTINCT COUNT(id) FROM master_tbl_view m
WHERE %(where)s"""

WHERE = """(%(start_date_column)s <= %%(end_date)s AND
%%(start_date)s <= coalesce(%(end_date_column)s, now()) AND
phase = ANY (%(successors)s)) """


class JobPercentage(object):

    def __init__(self):
        self.agency = Agency()

    def get_job_percentage(self):
        """
        Return the job percantage stored in the agency settings
        """
        return self.agency.getJobPecentage()

    def get_phases(self):
        """
        Returns phases to evaluate for job percentage
        """
        phases = g.mpuls_config.get('job_percentage', 'phases')
        if not phases:
            return []
        return [int(p_id) for p_id in phases.split(",")]

    def calulate_percentage(self, start=None, end=None):
        """Calculate all cases in phases divides thru job parts"""
        if start is None:
            start = date.today()
        if end is None:
            end = date.today()
        sm = g.case_factory.get_state_machine()
        phases = self.get_phases()
        where = "TRUE"
        log.debug("JobPercentage: phases %s" % phases)
        values_dict = {}
        if phases:
            where_buf = []
            for p_id in sm.get_all_state_ids():
                if p_id in phases:
                    fields = {'start_date_column': 'NOW()',
                              'end_date_column': 'NOW()'}
                    datefields = sm.get_datefield_names(p_id)
                    s_datefield = datefields[0]
                    e_datefield = datefields[1]
                    if s_datefield:
                        fields['start_date_column'] = s_datefield
                    if e_datefield:
                        fields['end_date_column'] = e_datefield
                    succs = sm.get_successor_state_ids(p_id)
                    values_dict[
                        'successors' + str(p_id)] = succs
                    values_dict['start_date'] = start
                    values_dict['end_date'] = end
                    fields['successors'] = \
                        '%(successors{key})s'.format(key=str(p_id))
                    where_buf.append(WHERE % fields)
            if where_buf:
                where = " OR ".join(where_buf)

        conn = db.getConnection()
        cur = conn.cursor()
        fields = {'where': where}
        sql_query = COUNT_ALL_CASES % fields
        log.debug(sql_query % values_dict)
        cur.execute(sql_query, values_dict)
        cases = cur.fetchone()[0]

        job_percentage = self.get_job_percentage()
        log.debug("JobPercentage: cases %d, jobparts %f" %
                  (cases, job_percentage))
        if job_percentage and cases:
            # cases = # of cases
            # stellenanteile (job percentage) of the agency
            percentage = cases / job_percentage

            return (cases, percentage)
        else:
            return (0, 0)
