#!/usr/bin/python
# -*- coding: utf-8 -*-
import sys
import psycopg2

CREATE_CASE = """
SET SESSION AUTHORIZATION %s;
SELECT create_master_tbl(NULL);"""

UPDATE_CASE = """
UPDATE master_tbl_view SET meta_entry = %s WHERE id = %s;
"""

UPDATE_STATUS = """
UPDATE ka_status_tbl_view SET status = 5 WHERE master_id = %s;
"""

ANONYMIZE_CASE = """
SET SESSION AUTHORIZATION postgres;
SELECT anonymize_case(%s);
"""


def create_all(port, dbname, meta, list1):
    ok, total = 0, 0
    id_list = []
    for entry in list1:
        (ok_new, id1) = create1(port, dbname, meta, entry)
        ok += ok_new
        total += 1
        if ok_new:
            id_list.append(str(id1))
    print "Created, updated and anonymize %d/%d users" % (ok, total)
    print "Following cases created: %s" % " ".join(id_list)

def create1(port, dbname, meta, entry):
    con = None
    try:
        con = psycopg2.connect(database='ka_' + dbname + '_db', port=port)
        db_user = 'ka_' + dbname + '_' + entry[0]
        cur = con.cursor()
        cur.execute(CREATE_CASE, (db_user,))
        id1 = int(cur.fetchone()[0])
        for (x,y) in zip(meta[1:],entry[1:]):
            sql = UPDATE_CASE.replace('meta_entry', x)
            cur.execute(sql, (y, id1))
        cur.execute(ANONYMIZE_CASE,(id1,))
        cur.execute(UPDATE_STATUS,(id1,))
        con.commit()
        return (1, id1)
    except psycopg2.DatabaseError, e:
        print "Error: %s" % e
    finally:
        if con:
            con.close()
    return (0, None)

def read_csv(f1):
    list1 =[]
    meta = []
    first_processed = False
    for line in f1:
        if not first_processed:
            meta = line.strip().split(',')
            first_processed = True
        elif line.strip():
            list1.append(line.strip().split(','))
    return meta,list1

def main():
    if len(sys.argv) -1 != 3:
        print "create_cases.py <port> <dbname> <csvlist>"
        print "Not 3 arguments -> exit"
        return
    port = sys.argv[1]
    dbname = sys.argv[2]
    input1 = open(sys.argv[3])
    meta,list1 = read_csv(input1)
    input1.close()
    print "Create new cases..."
    create_all(port, dbname, meta, list1)

main()
