#! /usr/bin/python
# -*- coding: utf-8 -*-
import json
import sys
import os
import traceback
import decimal
from optparse import OptionParser
from datetime import date, datetime

sys.path.append('/home/mpuls/jmd-auswertung/releases/current')
sys.path.append('/home/mpuls/jmd-auswertung/releases/current/libmpuls')
sys.path.append('/home/mpuls/jmd-auswertung/releases/current/base')
sys.path.append('/home/mpuls/jmd-auswertung/releases/current/libformed')

# For get_xml data
from libmpuls.evaluation.config import EvaluationConfig
from libmpuls.evaluation.evaluation import EvaluationSet
from libmpuls.evaluation.export import EvaluationExportXML
from libmpuls.db.db import get_db

# For parsing XML
import xml.etree.ElementTree as ET

# For writing xls
import xlwt

# For sending the emails.
import smtplib
from email import encoders
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText

# For cleaning files
import glob

###
# 1. Get fkzs for a carrier
# 2. get xmls for a fkz
# 3. Parse xmls
# 4. Create XSL
# 5. Send mail
###

### Constants to configure
WORKDIR="/tmp/auswertungen-fallakten"

def load_config(config_file_name):
    with open(config_file_name) as config_file:
        config = json.load(config_file)
    map(lambda x:set_global_variable(x[0], x[1]), config.iteritems())


def set_global_variable(key, value):
    globals()[key] = value



### Read the fkzs for a carrier from the db

def get_fkzs(db, carrier):
    # Takes a carrier and collects the fkzs for this carrier
    conn, cur = None, None
    try:
        conn = db.getConnection()
        if carrier in carrier_mapper:
            rows = []
            try:
                cur = conn.cursor()
                cur.execute('''SELECT fkz,
                               database,
                               name
                               FROM fkz_tbl
                               WHERE bundesland_id >= 0 AND
                                  fkz::integer/1000 = %s
                                  AND name NOT LIKE '%%(bis%%'
                               ORDER BY database;''',
                            (carrier_mapper[carrier]['case_nr'],))
                for row in cur:
                    rows.append((row[0], row[1], row[2]))
                # Restrict here for testing reasons e.g. rows[:3]
                return rows
            finally:
                db.recycleConnection(conn, cur)
    except Exception, err:
        #log.error('Error on evaluation set: %s' % err)
        traceback.print_exc()
        print "Failed to get connection. Check credetials"
    return []


### Gets the xmls and write them down.

def write_eval_to_tmp(db, year):
    for carrier in carrier_mapper:
        print "* Processing", carrier
        fkzs = get_fkzs(db, carrier)
        for (fkz, dbname, name) in fkzs:
            print "** fkz ", fkz
            for (ev, ev_path) in eval_mapper:
                print "*** evaluation", ev
                xml = get_xml(db, fkz, ev_path, year)
                with open('%s_%s_%s.xml' % (carrier, fkz, ev), 'wb') as f1:
                    f1.write(xml)


def get_xml(db, fkz, evaluation_path, year):
    # get the XML of an evaluation
    conn, cur = None, None
    try:
        try:
            conn = db.getConnection()
            config = EvaluationConfig(
                evaluation_path, 
                conn, 
                '%s-01-01' % year, 
                '%s-12-31' % year, 
                'datum_cm_start',
                'datum_cm_ende',
                '2,3',  # Start-phase
                ("select %(fields)s from master_tbl_eval_total_view m where m.id in (SELECT x.id from master_tbl_eval_total_view x join fkz_tbl f ON x.fkz::VARCHAR = f.id::VARCHAR WHERE f.fkz='"+ fkz +"')" ), 
                None,
                fkz,
                None,
                adele=True)
            evaluationset = EvaluationSet(config, False)
            evaluationset.evaluate()
            result = evaluationset.export(EvaluationExportXML())
            result_encoded = result.encode('utf-8')
            return result_encoded
        except Exception, err:
            traceback.print_exc()
            print "Failed to get connection. Check credetials"
    finally:
        db.recycleConnection(conn, cur)
        db.closeConnections()


### Code for parsing the xml

class EvaluationResult(object):
    def __init__(self, name, id1, fkz):
        self.name = name
        self.id1 = id1
        self.h1 = ['']
        self.h2 = ['']
        self.h3 = ['FKZ']
        self.h4 = ['']
        self.line = [fkz]

class ParsedEvaluation(object):
    def __init__(self, name):
        self.name = name
        self.lines = []

def parse_xml(db, carrier, ev):
    # parse the xmls for one evaluationset
    start = True # at the beginning we want a list of ParsedEvaluations
    results = []
    for (fkz, dbname, name) in get_fkzs(db, carrier):
        for idx, input1 in enumerate(parse_xml_of_one_file(carrier, fkz, ev)):
            if start:
               parsed_evaluation = ParsedEvaluation(
                                     "(%s)_%s" % (input1.id1, ev))
               parsed_evaluation.lines.append(input1.h1)
               parsed_evaluation.lines.append(input1.h2)
               parsed_evaluation.lines.append(input1.h3)
               parsed_evaluation.lines.append(input1.h4)
               results.append(parsed_evaluation)
            results[idx].lines.append(input1.line)
        start = False
    return results


def parse_xml_of_one_file(carrier, fkz, ev):
    tree = ET.parse("%s_%s_%s.xml" % (carrier, fkz, ev))
    doc = tree.getroot()
    results = []
    for evaluation in doc.findall('evaluation'):
        result = EvaluationResult(ev, evaluation.attrib['id'], fkz)
        result.h1.append(evaluation.attrib['name'])
        for table in evaluation.findall('table'):
            result.h2.append(table.attrib['ylabel'])
            headers = [header.text for header in table.findall('headers/header')]
            for row in table.findall('rows/row'):
                result.h3.append(row.attrib['label'])
                for index, col in enumerate(row.findall('col')):
                    result.h4.append(headers[index % len(headers)])
                    result.line.append(col.text)

                # update header 3
                while len(result.h3) < len(result.h4):
                    result.h3.append('')

                # update header1
                while len(result.h1) < len(result.h4):
                    result.h1.append('')

                # update header2
                while len(result.h2) < len(result.h4):
                    result.h2.append('')

        results.append(result)
    return results


### Code for writing the parsed data into the xls

def write_to_xls(db):
    filenames = {}
    print "* Generating xls"
    for carrier in carrier_mapper:
        print "** ", carrier
        wb = xlwt.Workbook()
        for (ev, ev_path) in eval_mapper:
            for pe in parse_xml(db, carrier, ev):
                ws = wb.add_sheet(pe.name)
                if len(pe.lines[0]) > 255:
                    cutting = []
                    lines = pe.lines
                    while len(lines[0]) > 255:
                        cut1 = [line[0:255] for line in lines]
                        lines = [line[255:] for line in lines]
                        cutting.append(cut1)
                    if len(lines[0]) > 0:
                        cutting.append(lines)
                    for k, cut in enumerate(cutting):
                        sheet_name = pe.name + "_" + str(k + 1)
                        ws = wb.add_sheet(sheet_name)
                        write_to_sheet(ws, cut)
                else:
                    write_to_sheet(ws, pe.lines)
        filename = "%s-cm-%s.xls" % (carrier, date.today().isoformat())
        wb.save(filename)
        filenames[carrier] = filename
    return filenames


def write_to_sheet(ws, lines):
    for (rownr, row) in enumerate(lines):
        for (colnr, col) in enumerate(row):
            try:
                out = decimal.Decimal(col)
            except decimal.InvalidOperation:
                out = col
            ws.write(rownr, colnr, out)


### Code for sending the mails

text_template = u"""\
Anbei erhalten Sie die Auswertungen 'Analyse', 'CM' und 'Zielgruppe'
für den Zeitraum  01.01.%(year)s bis zum 31.12.%(year)s als xls-Datei.

Viele Grüße
Ihr mpuls-Team
"""

def send_mail(carrier, emails, xls_filename, options):
    print "* send_mails", carrier 
    outer = MIMEMultipart()
    outer['Subject'] = 'Auswertungen JMD Einzellfallakte (CM)'
    outer['To'] = ', '.join(emails)
    outer['From'] = 'mpuls@intevation.de'

    outer.attach(MIMEText(text_template % {"year": options.year},
                          _charset="latin1"))

    with open(xls_filename) as fp:
        msg = MIMEBase('application', 'octet-stream')
        msg.set_payload(fp.read())
    encoders.encode_base64(msg)

    msg.add_header('Content-Disposition', 'attachment', filename=xls_filename)
    outer.attach(msg)

    # Delete the files
    if not options.keep_files:
        os.remove(xls_filename)
        for f1 in glob.glob('%s*.xml' % carrier):
            os.remove(f1)

    composed = outer.as_string()
    # Write to disk or send per email
    if options.send_email:
        s = smtplib.SMTP('localhost')
        recipients = emails
        if options.forced_recipient is not None:
             recipients = [options.forced_recipient]
        s.sendmail('mpuls@intevation.de', recipients, composed)
        s.quit()
    else:
        with open('tmp-email-%s.txt' % carrier, 'w') as fp:
            fp.write(composed)

# Main function

def main():
    #
    # Parse user options
    #
    parser = OptionParser()
    parser.add_option("--year",
                  action="store",
                  dest="year",
                  default=date.today().year,
                  help="Year to filter the reports. Default: current year")
    parser.add_option("--dont-send-email",
                  dest="send_email",
                  action="store_false",
                  default=True,
                  help=("Don't send emails with the reports but write them"
                        " to a file instead 'tmp-email-$TRAEGER.txt'"))
    parser.add_option("--keep-files",
                  action="store_true",
                  dest="keep_files",
                  default=False,
                  help="Don't delete generated files")
    parser.add_option("--force-recipient",
                  action="store",
                  dest="forced_recipient",
                  default=None,
                  help=("Send mails only to this address for"
                        " for easier testing"))
    parser.add_option("--config",
                  action="store",
                  dest="config_file",
                  default=None,
                  help="Configuraton file - mandatory")

    (options, args) = parser.parse_args()

    if not options.config_file:
        print("Configuration file missing. Aborting.")
        exit(2)
    else:
        load_config(options.config_file)
    
    print "\n** Start %s" % datetime.now()
    # Create db with user infos
    db = get_db(USERNAME, PASSWORD, DBNAME, HOST, PORT)
    
    # Main stuff
    write_eval_to_tmp(db, options.year)
    xls_filenames = write_to_xls(db)
    for carrier in carrier_mapper:
        send_mail(carrier, carrier_mapper[carrier]['emails'],
                  xls_filenames[carrier], options)

    print "** End %s" % datetime.now()

# Test code

if __name__=='__main__':
    if not os.access(WORKDIR,os.F_OK):
        os.mkdir(WORKDIR)

    os.chdir(WORKDIR)
    main()
