#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Generate an XLS file from a po-file.
Usage:
gettext-to-xlwt.py  PO_FILE  XLS_FILE
"""

import sys
import optparse

import xlwt
from babel.messages.pofile import read_po


def list_po_file(filename):
    catalog = read_po(open(filename))
    for msg in catalog:
        if msg.id == "":
            continue
        value = msg.string
        if msg.fuzzy:
            if msg.pluralizable:
                value = ("", "")
            else:
                value = ""

        comments = list(msg.auto_comments)

        if "\n" in msg.id:
            comments.insert(0, "mehrzeilig")

        comments = ";".join(comments)

        if msg.pluralizable:
            yield (msg.id[0], value[0], comments)
            yield (msg.id[1], value[1], "")
        else:
            yield (msg.id, msg.string, comments)


def make_workbook(translations):
    workbook = xlwt.Workbook()
    sheet = workbook.add_sheet(u"Übersetzungen")
    sheet.write(0, 0, u"Schlüssel")
    sheet.write(0, 1, u"Übersetzung")
    sheet.write(0, 2, u"Kommentare")

    style = xlwt.XFStyle()

    for row_num, (key, value, comments) in enumerate(translations):
        if isinstance(key, tuple):
            key = key[0]
        sheet.write(row_num + 1, 0, key, style)
        sheet.write(row_num + 1, 1, value, style)
        sheet.write(row_num + 1, 2, comments, style)

    return workbook

def write_workbook(workbook, filename):
    workbook.save(open(filename, "w"))


def po_to_xls(filename, xls_filename):
    write_workbook(make_workbook(list_po_file(filename)), xls_filename)


def main():
    usage = "Usage: %prog [options] PO-FILE XLS-FILE"
    # we don't really need to parse options, but using OptionParser
    # gives us standard behaviour, like a --help option, for free.
    parser = optparse.OptionParser(usage=usage)
    options, args = parser.parse_args()

    if len(args) != 2:
        print >>sys.stderr, ("The po-file and the xls-file must be given"
                             " as command line arguments")

    po_to_xls(args[0], args[1])


main()
